#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include <stdio.h>
#include <tcl.h>
#include <tk.h>

#include "client.h"
#include "IBRCClient.h"
#include "common/types.h"

#define SCRIPT "IBRC.tcl"

/* Tk Hauptfenster */
Tk_Window  mainWindow;
Tcl_Interp *interpreter;
char username[IBRC_NAMELEN];

int setVar(char *variable, char *value){
  char *varValue = Tcl_SetVar(interpreter, variable, value, TCL_LEAVE_ERR_MSG);

  if(varValue == NULL){
    return TCL_ERROR;
  }
  else{
    interpreter->result = varValue;
    return TCL_OK;
  }
}

int printLine(char *line){
  int result;
  char command[256];

  bzero(command, 256);
  sprintf(command, ".text insert end \"%s\\n\"\n.text yview end", line);
  
  result = Tcl_Eval(interpreter, command);

  if(result != TCL_OK){
    return -1;
  }
  else{
    return 1;
  }  
}

void setName(char *name){
  strncpy(username, name, IBRC_NAMELEN);
  setVar("REALNAME", name);
}

void setNick(char *nick){
  setVar("NICKNAME", nick);
}

void setChannel(char *channel){
  setVar("CHANNEL", channel);
}

void printSystemMessage(char *message){
  char line[1024];

  bzero(line, 1024);
  sprintf(line, "%s\n", message);
  printLine(line);
}

void printOwnMessage(char *message){
  char line[1024];

  bzero(line, 1024);
  sprintf(line, "%s: %s\n", username, message);
  printLine(line);
}

void printMessage(char *from, char *message){
  char line[1024];

  bzero(line, 1024);
  sprintf(line, "%s: %s\n", from, message);
  printLine(line);
}

void printPrivMessage(char *from, char *message){
  char line[1024];

  bzero(line, 1024);
  sprintf(line, "%s> %s\n", from, message);
  printLine(line);
}

void bye(){
  /*printf("ENDE\n");*/
   
  quit();   
  exit (1);
}

int quitIBRC(ClientData clientData, Tcl_Interp *interp, int argc, char **argv){
  /*printf("ENDE\n");*/
   
  quit();   
  exit (1);
  /*return TCL_OK;*/
}

int newInput(ClientData clientData, Tcl_Interp *interp, int argc, char **argv){
  if(argc == 2){
    userInput(argv[1]);
  }

  return TCL_OK;
}

/*void endSession(){
  exitIBRCClient();
  printf("The end.\n");
  exit(1);
}*/

void showMainloop(void *clientdata){
  mainloop();
  Tk_CreateTimerHandler(1, &showMainloop, NULL);
}

int Tk_AppInit(Tcl_Interp *interp){
  if(Tcl_Init(interp) != TCL_OK){
    fprintf(stderr, "Tcl_Init failed: %s\n", interp->result);
    exit(1);
  }

  if(Tk_Init(interp) != TCL_OK){
    fprintf(stderr, "Tk_Init failed: %s\n", interp->result);
    exit(1);
  }

  Tcl_CreateCommand(interp, "new_input", newInput, NULL, NULL);
  Tcl_CreateCommand(interp, "quit_ibrc", quitIBRC, NULL, NULL);
  
  interpreter = interp;
  mainWindow  = Tk_MainWindow(interp);

  Tk_UnmapWindow(mainWindow);
  Tk_CreateTimerHandler(1, &showMainloop, NULL);
  Tk_MapWindow(mainWindow);

  return TCL_OK;
}

int main(int argc, char **argv){
  char *tkArgs[2];

  if(argc != 3){
    fprintf(stderr, "Usage: %s <server> <port>\n", argv[0]);
    exit(1);
  }

  tkArgs[0] = malloc(strlen(argv[0])+1);
  tkArgs[1] = malloc(strlen(SCRIPT)+1);

  strcpy(tkArgs[0], argv[0]);
  strcpy(tkArgs[1], SCRIPT);  

  signal(SIGTERM, bye);
  signal(SIGINT,  bye);

  init(argv[1], atoi(argv[2]));

  Tk_Main(2, tkArgs, Tk_AppInit);
  
  exit(0);
}
