#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h> /* open, close Fd */
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <netdb.h>

#include "common/types.h"
#include "common/command.h"
#include "common/util.h"

#include "IBRCClient.h"
#include "client.h"

const char* ibrc_ccmds[IBRC_CLIENTCMDS_COUNT]=
  {
    "JOIN",
    "LEAVE",
    "NAME",
    "NICK",
    "LIST",
    "GETTOPIC",
    "SETTOPIC",
    "PRIVMSG",
    "QUIT"
  };
const char coch_IBRC_CMDSTART='/';


int nSocketFd, nMaxSock, nReadySocks, cnt;
char cstr[IBRC_MAXSTRSIZE], cstrNick[IBRC_NICKLEN], cstrName[IBRC_NAMELEN], cstrChannel[IBRC_CHANNELLEN];
fd_set readset, tmpset;
struct timeval timeout;
 
void evaluateCommand(char *);

void interpreteServerMessage(char *);


void init(char *server, int port){
  printf("Connecting Server %s on port %d ...\n",server,port);
  nSocketFd = connectHost(server, port);
  /* Set the "sets" */
  FD_ZERO(&readset);
  FD_SET(nSocketFd, &readset);
  nMaxSock = nSocketFd;
  /* timeout.tv_sec = 1; */
  timeout.tv_usec = 500; 
}



void quit(){
  close(nSocketFd);
}



void userInput(char *input){
  char cstrUsrMsg[IBRC_MAXSTRSIZE], msg[IBRC_MAXSTRSIZE], *cstrToken;
  strncpy(cstrUsrMsg, input, IBRC_MAXSTRSIZE);
  printf("user: %s\n",cstrUsrMsg);
  cstrToken=strtok(input, ibrc_DELIMS);
  
  if(cstrToken!=NULL){
    /* starting with Command-Start-Sign, evaluate Command */
    if(coch_IBRC_CMDSTART==cstrToken[0]){
      ++cstrToken;
      evaluateCommand(cstrToken);
    }
    /* no command, simple input: */
    else{
      printOwnMessage(cstrUsrMsg);
      sprintf(msg,"MSG %s %s %s\n",cstrNick, cstrChannel, cstrUsrMsg);
      /* write message to the server */
      write(nSocketFd, msg, strlen(msg));
    }
  }
}



void mainloop(){
  /* Copy readset into the temporary set */
  memcpy(&tmpset, &readset, sizeof(readset));
  /* Wait indefinitely for read events */
  cnt = select(nMaxSock+1, &tmpset, NULL, NULL, &timeout);
  
  if (nReadySocks < 0) {
    perror("Error Processing.");
  }
  /* handle received data */
  if (FD_ISSET(nSocketFd, &tmpset)) {
    /* handle data from server socket */
    cnt = recv(nSocketFd, cstr, IBRC_MAXSTRSIZE, 0);
    if (cnt <= 0) {
      /* Closed connection or error */
      FD_CLR(nSocketFd, &readset);
      /* should check nMaxSock here ... */
      /* do some cleaning */
      shutdown(nSocketFd, 2);
      close(nSocketFd);
      sprintf(cstr,"Lost server connection.");
      printSystemMessage(cstr);
    }
    else {
      /* Process data in buffer */
      interpreteServerMessage(cstr);
    }
  }
}



void evaluateCommand(char* cstrToken){
  char *cstrJoin;
  if(0==strcmp(cstrToken,ibrc_ccmds[IBRC_CCMD_JOIN])){
    cstrToken=strtok(NULL,cstrToken); /* Channel to join */
    strncpy(cstrChannel, cstrToken, IBRC_CHANNELLEN);
    setChannel(cstrChannel);
    printf("Entering Channel %s.\n",cstrChannel);
    cstrJoin = (char *)malloc(sizeof("JOIN  \n ")+strlen(cstrNick)+strlen(cstrToken));
    sprintf(cstrJoin, "JOIN %s %s\n",cstrNick, cstrToken);
    write(nSocketFd, cstrJoin, strlen(cstrJoin));
    free(cstrJoin);
  }
  else if(0==strcmp(cstrToken,ibrc_ccmds[IBRC_CCMD_LEAVE])){
    printf("Leaving Channel.\n");
  }
  else if(0==strcmp(cstrToken,ibrc_ccmds[IBRC_CCMD_NAME])){
    cstrToken=strtok(NULL, ibrc_DELIMS);
    printf("Changing Name to %s.\n",cstrToken);
    if(cstrToken!=NULL){
      setName(cstrToken);
    }
  }
  else if(0==strcmp(cstrToken,ibrc_ccmds[IBRC_CCMD_NICK])){
    cstrToken=strtok(NULL, ibrc_DELIMS);
    if(cstrToken!=NULL){
      setNick(cstrToken);
      strncpy(cstrNick, cstrToken, IBRC_NICKLEN);
    }
    printf("Changing Nickname to %s.\n",cstrNick);
  }
  else if(0==strcmp(cstrToken,ibrc_ccmds[IBRC_CCMD_LIST])){
    printf("Listing Channels.\n");
  }
  else if(0==strcmp(cstrToken,ibrc_ccmds[IBRC_CCMD_GETTOPIC])){
    printf("Getting Channel Topic.\n");
  }
  else if(0==strcmp(cstrToken,ibrc_ccmds[IBRC_CCMD_SETTOPIC])){
    printf("Setting Channel Topic.\n");
  }
  else if(0==strcmp(cstrToken,ibrc_ccmds[IBRC_CCMD_PRIVMSG])){
    printf("Sending private Message.\n");
  }
  else if(0==strcmp(cstrToken,ibrc_ccmds[IBRC_CCMD_QUIT])){
    printf("Exit. Bye!\n");
    bye();
  }
  else {
    printf("Unknown Command: %s!\n",cstrToken);
  }
}

void interpreteServerMessage(char * cstr){
  char cstrSrvMsg[IBRC_MAXSTRSIZE], *nick, *channel, *cstrToken;
  strncpy(cstrSrvMsg, cstr, IBRC_MAXSTRSIZE);
  printf("server: %s\n",cstrSrvMsg);
  cstrToken=strtok(cstr, ibrc_DELIMS);
  
  if(cstrToken!=NULL){
    if(0==strcmp(cstrToken, ibrc_commands[IBRC_COMMAND_MSG])){
      cstrToken=strtok(NULL, ibrc_DELIMS); /* nick */
      nick = cstrToken;
      cstrToken=strtok(NULL, ibrc_DELIMS); /* channel */
      channel = cstrToken;
      cstrToken=strtok(NULL, ibrc_MSGEND); /* message */
      printMessage(nick, cstrToken); /* print to UI */
    }
    else if(0==strcmp(cstrToken, ibrc_commands[IBRC_COMMAND_JOIN])){
      /* ... */
    }
    else if(0==strcmp(cstrToken, ibrc_commands[IBRC_COMMAND_LEAVE])){
      /* ... */
    }
    else if(0==strcmp(cstrToken, ibrc_commands[IBRC_COMMAND_PRIVMSG])){
      /* ... */
    }
    else if(0==strcmp(cstrToken, ibrc_commands[IBRC_COMMAND_NICKCHANGE])){
      /* ... */
    }
    else if(0==strcmp(cstrToken, ibrc_commands[IBRC_COMMAND_TOPIC])){
      /* ... */
    }
    else if(0==strcmp(cstrToken, ibrc_commands[IBRC_COMMAND_ERROR])){
      /* ... */
    }
    else if(0==strcmp(cstrToken, ibrc_commands[IBRC_COMMAND_LISTREPLY])){

      /* ... */
    }
    else{
      printSystemMessage("(Unknown server command.)");
    }
  }
  
}
