/*===========================================================
 FILENAME     command.c
 VERSION      $Rev$

 DESCRIPTION  names for commands
 AUTHOR       phk

 CHANGES
 Date       | Name             | Comment
 ------------------------------------------------------------
 2004-05-13 | phk              | Initial Version
 
=============================================================*/

#include "command.h"


#define  _GNU_SOURCE
#include <string.h>
#include <stdlib.h>

char* ibrc_commands[IBRC_COMMANDS_COUNT] = {
	/* server commands */
	"CONNECT",
	"SERVERROR",
	"ERROR",

	/* channel commands */
	"JOIN",
	"LEAVE",
	"MSG",
	"NICKCHANGE",
	"TOPIC",
	"GETTOPIC",
	"LIST",
	"LISTREPLY",

	/* private command */
	"PRIVMSG"
};

char *ibrc_DELIMS=" ";
char *ibrc_MSGEND="\n";


/* better be on the safe side */
#define MAX_PARAMS IBRC_MAXSTRSIZE / 2

/** buffer must be zero-terminated,
 *  caller retains buffer ownership */
void command_generic_parse(const char* buffer) {
	trCommand* cmd;
	
	char *search, *end , *copy1, *copy2;
	char *i;

	int j, param = 0, nrparams;
	char *params[MAX_PARAMS];

	/* protocol specifies network line endings (CRLF) IMHO,
	 * but better be safe than sorry. */
	end = strchr(buffer, '\r');
	search = strchr(buffer, '\n');
	if ((search < end) || (end == NULL))
		end = search;
	if (end == NULL) {
		/* ERROR: line not terminated */
		return;
	}

	cmd = malloc(sizeof(trCommand));
	memset(cmd, 0, sizeof(trCommand));

	copy1 = strndup(buffer, (size_t)(end-buffer));
	copy2 = strndup(buffer, (size_t)(end-buffer));

	cmd->_full_cmdstr = copy1;
	cmd->_split_cmdstr = copy2;


	for (i=copy2; *i!='\0'; i++) {
		if ((*i == ' ') || (*i =='\t')) {
			while ((*i == ' ') || (*i =='\t')) {
				*i = '\0';
				i++;
			}
			if (*i == '\0') {
				break;
			}

			params[param] = i;
			if (param > MAX_PARAMS){
				/* this is an error ! */
				return;
			}
		}
	}

	/* Now let's try to identify params[0]... */
	for (j=IBRC_COMMAND_CONNECT; j<IBRC_COMMANDS_COUNT; j++) {
		if (strcmp(ibrc_commands[j], params[0]) == 0){
			goto found;
		}
	}
	/* no match... protocol error */
	return;

found:
	cmd->command = j;

	switch (cmd->command) {
		case IBRC_COMMAND_CONNECT:
		case IBRC_COMMAND_SERVERROR:
		case IBRC_COMMAND_ERROR:
			/* server commands */
			nrparams = 0;
			break;

	/* channel commands */
		case IBRC_COMMAND_JOIN:
		case IBRC_COMMAND_LEAVE:
		case IBRC_COMMAND_MSG:
		case IBRC_COMMAND_NICKCHANGE:
		case IBRC_COMMAND_TOPIC:
		case IBRC_COMMAND_GETTOPIC:
		case IBRC_COMMAND_LIST:
		case IBRC_COMMAND_LISTREPLY:
			break;

	/* private command */
		case IBRC_COMMAND_PRIVMSG:
			break;
	}

	/*
	cmd->params._raw[param] = i;

	param++;
	if (param > IBRC_COMMANDS_MAX_PARAM_GROUPS){
		// no need to split further
		break;
	}
	*/
}

/*
typedef struct rCommand{
	int				command;

	int				rcvd_on_fd;

	trClient*			client;
	trChannel*			channel;

	union {
		struct {
			char*		cmdstr;
			char*		name;
			char*		errormsg;
		} servercmd;
		struct {
			char*		cmdstr;
			char*		nick;
			char*		channel;
			char*		argument;
		} channelcmd;
		struct {	
			char*		cmdstr;
			char*		nick;
			char*		channel;
			char*		recipient;
			char*		argument;
		} privatecmd;
			
		char*			_raw [IBRC_COMMANDS_MAX_PARAM_GROUPS];
	} params;

	char*				_full_cmdstr;
	char*				_mixed_cmdstr;
} trCommand;
*/
