/*===========================================================
 FILENAME     protocol.h
 VERSION      0.1

 DESCRIPTION  Basic IBRC Types and IBRC Protocol format
 AUTHOR       Sonnenberg

 CHANGES
 Date       | Name             | Comment
 ------------------------------------------------------------
 2004-06-04 | Sonnenberg       | Initial Version
 
=============================================================*/

#ifndef __PROTOCOL_H
#define __PROTOCOL_H

/*----- General defines -----*/
#define IBRC_NICKLEN            9
#define IBRC_TOPICLEN           13
#define IBRC_MAXBUFFSIZE        512
#define IBRC_MAXSTRSIZE         80
#define IBRC_MAXNUMOFSERVER     10
#define IBRC_MAXNUMOFCLIENTS    20
#define IBRC_MAXNUMOFCHANNELS   100

/*----------------------*/
/*----- IBRC Types -----*/
/*----------------------*/

/** IBRC Command */
typedef enum enIBRC_Cmd{
  /** No Command */
  IBRC_CMD_UNDEFINED = 0,

  /*----- Client Commands ----- */
  /** Connect to a server */
  IBRC_CCMD_CONNECT = 1,
  /** Join a channel */
  IBRC_CCMD_JOIN = 2,
  /** Leave a channel */
  IBRC_CCMD_LEAVE = 3,
  /** Set/Change Nickname */
  IBRC_CCMD_NICK = 4,
  /** Order List of ex. Channels */
  IBRC_CCMD_LIST = 5,
  /** Get Topic of Channel */
  IBRC_CCMD_GETTOPIC = 6,
  /** Set/Change Topic of Channel */
  IBRC_CCMD_SETTOPIC = 7,
  /** Send Message to Channel (one-to-many) */
  IBRC_CCMD_CHMSG = 8,
  /** Send Message to Client (one-to-one) */
  IBRC_CCMD_PRIVMSG = 9,
  /** Server to Client Reply */
  IBRC_CCMD_REPLY = 10,
  /** Quit server */
  IBRC_CCMD_QUIT = 11,

  /*----- Server Commands -----*/
  /** Connect to other server */
  IBRC_SCMD_CONNECT = 12,
  /** Introduce/Change Client's Nick */
  IBRC_SCMD_NICK = 13,
  /** Client specific changes (Nick, Channel, ...) */
  IBRC_SCMD_CLIENTCHAN = 14,
  /** Delete Client */
  IBRC_SCMD_DELCLIENT = 15,
  /** Introduce/Change Channel */
  IBRC_SCMD_CHANNEL = 16,
  /** Deliver Channel-Message */
  IBRC_SCMD_CHMSG = 17,
  /** Deliver Client-Client Message */
  IBRC_SCMD_PRIVMSG = 18,
  /** Server-quit or unavailability notice */
  IBRC_SCMD_QUIT = 19
} tenIBRC_Cmd;

/** IBRC Reply*/
typedef enum enIBRC_Reply{
  /** Affirmation Reply */
  IBRC_RPL_OK = 0,
  /** Error Reply */
  IBRC_RPL_ERROR = 1,
  
  /*----- Server to Client Replies -----*/
  /** Connect Reply */
  IBRC_CRPL_CONNECT = 2,
  /** Channel List */
  IBRC_CRPL_LIST = 3,
  /** Get topic information */
  IBRC_CRPL_TOPICINFO = 4,
  /*----- Server to Server Replies -----*/
  /** Connect Reply */
  IBRC_SRPL_CONNECT = 5
} tenIBRC_Reply;


/** IBRC Error*/
typedef enum enIBRC_Error{
  /** Unknown Error */
  IBRC_ERR_UNKNOWN = 0,
  /** More Parameters needed */
  IBRC_ERR_NEEDMOREPARAMS = 1,
  /** Password mismatch */
  IBRC_ERR_PASSWDMISMATCH = 2,
  /** Client unknown */
  IBRC_ERR_NOSUCHCLIENT = 3,

  /*----- Client-Server related Errors -----*/
  /** Nickname already in use */
  IBRC_ERR_NICKINUSE = 4,
  /** Client already connected with a server */
  IBRC_ERR_ALREADYCONNECTED = 5,
  /** Channel operator privileges needed */
  IBRC_ERR_NOTCHANOP = 6,

  /*----- Server-Server related Errors -----*/
  /** Channel unknown */
  IBRC_ERR_NOSUCHCHANNEL = 7,
  /** Seerver unknown */
  IBRC_ERR_NOSUCHSERVER = 8,
  /** Max. number of opened channels exceeded */
  IBRC_ERR_TOOMANYCHANNELS = 9
} tenIBRC_Error;


#endif /* __PROTOCOL_H */
