#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "hashtable.h"

trHashtable *hash_table_insert(trHashtable *hash_table[], char *key, void *val)
{
  trHashtable *pNewEntry;
  /* get hashtable-adress for hashvalue (bucket) */
  unsigned int hash_adress = hash_function(key);
  /* pointer to hashtable-adress calculated by hash_function*/
  pNewEntry = hash_table[hash_adress];
  /* Search position for new bucket in hash_table[hash_adress] */
  while(pNewEntry != NULL){
    if(strcmp(key, pNewEntry->key) == 0){  /* same key */
      /* avoid double entries (same value): */
      fprintf(stderr,"Element with hashkey %s already inserted!\n", key);
      return pNewEntry;
    }
    pNewEntry=pNewEntry->next;
  }
  
  /* allocate memory for new hashtable entry and insert in front of list */
  pNewEntry = (trHashtable *)malloc(sizeof(trHashtable));
  if(pNewEntry == NULL){
    fprintf(stderr, "Error allocating memory for new hashtable entry!\n");
    return NULL;
  }
  /* set key and value of new entry */
  strcpy(pNewEntry->key, key);
  pNewEntry->val = val;
  /* insert new entry in front of list at hash_table[hash_adress] */
  /* set pointers of new element */
  pNewEntry->prev = NULL;
  pNewEntry->next = hash_table[hash_adress];
  /* update previous-pointer of old first element in list hash_table[hash_adress] */
  if(hash_table[hash_adress]!=NULL)
    hash_table[hash_adress]->prev = pNewEntry;
  /* insert new entry */
  hash_table[hash_adress] = pNewEntry;
  /* printf("Inserted %s\n", pNewEntry->key); */
  return pNewEntry;
}

/* Lookup function */
void *hash_table_lookup(trHashtable *hash_table[], char *key){
  trHashtable *pEntry = private_hash_lookup(hash_table, key);
  if(pEntry!=NULL) return pEntry->val;
  else return NULL;
}

trHashtable *private_hash_lookup(trHashtable *hash_table[], char *key){
  trHashtable *pEntry;

  /* Get hash_adress */
  unsigned int hash_adress = hash_function(key);
  /* Get hashtable-list for hash_adress */
  pEntry = hash_table[hash_adress];
  /* Search that list for key */
  while(pEntry != NULL){
    if(strcmp(pEntry->key, key) == 0)	break;
    pEntry = pEntry->next;
  }
  return pEntry;
}


void hash_table_delEntry(trHashtable *hash_table[], char *key){
  trHashtable *pEntry2del = private_hash_lookup(hash_table, key);
  if(pEntry2del==NULL) fprintf(stderr,"hash_table_delEntry: No such entry: %s!\n",key);
  else{
    /* printf("Deleting %s\n", pEntry2del->key); */
    if(pEntry2del->prev!=NULL) pEntry2del->prev->next=pEntry2del->next;
    else hash_table[hash_function(key)]=NULL; /*prev NULL: first entry, set list NULL */
    if(pEntry2del->next!=NULL) pEntry2del->next->prev=pEntry2del->prev;
    free(pEntry2del);
  }
}

void hash_table_renameEntry(trHashtable *hash_table[], char *oldKey, char* newKey){
  void *tmpVal;
  trHashtable *pEntry2Rename = private_hash_lookup(hash_table,oldKey);
  if(pEntry2Rename!=NULL){
    tmpVal=pEntry2Rename->val;
    hash_table_delEntry(hash_table, oldKey);
    hash_table_insert(hash_table, newKey, tmpVal);
  }
  else fprintf(stderr,"hash_table_renameEntry: No such entry: %s!\n",oldKey);
}

void hash_table_changeValue(trHashtable *hash_table[], char *key, void *newVal){
  trHashtable *pEntry = private_hash_lookup(hash_table, key);
  pEntry->val=newVal;
}



/* hash-function to calculate hash-value for a cstring */
unsigned int hash_function(char *cstr)
{
  unsigned int hash_adress;
  unsigned char *pChar;
  
  hash_adress=0;
  pChar = (unsigned char *)cstr;
  
  while(*pChar != '\0'){
    hash_adress = HASHVALUE * hash_adress + *pChar;
    pChar++;
  }
  return hash_adress % HASHTABLESIZE;
}

void hash_table_free_values(trHashtable *hash_table[]){
  int i;
  trHashtable *pEntry;
  for(i=0; i<HASHTABLESIZE; ++i){
    pEntry = hash_table[i];
    if(pEntry!=NULL){
      do{
	if(pEntry->val) free(pEntry->val); 
	pEntry=pEntry->next;
      } while(pEntry != NULL);
    }
  }
}

void hash_table_free(trHashtable *hash_table[]){
  int i;
  trHashtable *pEntry, *pTmp;
  for(i=0; i<HASHTABLESIZE; ++i){
    pEntry = hash_table[i];
    if(pEntry!=NULL){
      do{
	pTmp=pEntry->next;
	/* printf("Freeing %s in list %i\n", pEntry->key, i); */
	free(pEntry);
	pEntry=pTmp;
      } while(pEntry != NULL);
    }
  }
}

void hash_table_free_inclKeys(trHashtable *hash_table[]){
  int i;
  trHashtable *pEntry, *pTmp;
  for(i=0; i<HASHTABLESIZE; ++i){
    pEntry = hash_table[i];
    if(pEntry!=NULL){
      do{
	pTmp=pEntry->next;
	/* printf("Freeing %s in list %i\n", pEntry->key, i); */
	free(pEntry->key);
	free(pEntry);
	pEntry=pTmp;
      } while(pEntry != NULL);
    }
  }
}
