#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h> /* open, close Fd */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include "common/types.h"
#include "common/command.h"
#include "common/util.h"
#include "hashtable.h"

void interpreteCommand(char *, int);
void addClient(char *, trChannel *, int);
trChannel *addChannel(char *);
void sendToChannel(char *, char *, int from);
void sendToAll(char *, int from);

trServer rParentServer;
trHashtable *htChildserver[HASHTABLESIZE];
trHashtable *htChannels[HASHTABLESIZE];
trHashtable *htClients[HASHTABLESIZE];

int main(int argc, char **argv){
  /* variables */
  bool bClientConnected=false;
  int i, nMainSocketFd, nNewSocketFd, nAddrLength, nPort, nMaxSock, nReadySocks, nLength;
  char cstr[IBRC_MAXSTRSIZE];
  fd_set readset, tempset;
  struct sockaddr_in rAddr;

  /* initialization */
  for(i=0; i<HASHTABLESIZE; ++i) htChildserver[i]=NULL;
  for(i=0; i<HASHTABLESIZE; ++i) htChannels[i]=NULL;
  for(i=0; i<HASHTABLESIZE; ++i) htClients[i]=NULL;
  
  /* get the port number */
  if(2!=argc && 4!=argc){
    printf("Usage: %s port [parenthost port]\n",argv[0]);
    exit(1);
  }
  else if(2==argc){
    nPort=atoi(argv[1]);
  }
  else if(4==argc){
    nPort=atoi(argv[1]);
    rParentServer.nSockFd = connectHost(argv[2],atoi(argv[3]));
  }
  
  /* open TCP socket */
  if((nMainSocketFd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
    perror("Can't open socket! Exit.");
    exit(1);
  }
  
  /* bind local address so that the client can connect */
  bzero((char*) &rAddr, sizeof(rAddr));
  rAddr.sin_family = AF_INET;
  rAddr.sin_addr.s_addr = htonl(INADDR_ANY);
  rAddr.sin_port = htons(nPort);
  
  if(bind(nMainSocketFd, (struct sockaddr *) &rAddr, sizeof(rAddr)) < 0) {
     perror("Error binding local address");
     exit(1);
  }

  /* listen to the socket */
  listen(nMainSocketFd, 5);
  printf("Connected and waiting.\n");

  /* Set the "sets" */
  FD_ZERO(&readset);
  FD_SET(nMainSocketFd, &readset);
  nMaxSock = nMainSocketFd;
  
  while(!0) {
    /* Copy readset into the temporary set */
    memcpy(&tempset, &readset, sizeof(readset));
    /* Wait indefinitely for read events */
    nReadySocks = select(nMaxSock+1, &tempset, NULL, NULL, NULL);
    if (nReadySocks < 0) {
      fprintf(stderr,"Error Processing.");
      break;
    }
    if (nReadySocks == 0) {
      printf("This should not happen without using timeout!?!?\n");
      continue;
    }
    
    /* handle new connections */
    if (FD_ISSET(nMainSocketFd, &tempset)) {
      /* New connection, handle it */
      nAddrLength = sizeof(rAddr);
      nNewSocketFd = accept(nMainSocketFd,(void *)&rAddr, &nAddrLength);
      printf("New Connection established. Listening.\n");
      /* check the result here... */
      FD_SET(nNewSocketFd, &readset);
      if (nNewSocketFd > nMaxSock)
	nMaxSock = nNewSocketFd;
      FD_CLR(nMainSocketFd, &tempset); /* don't read from the server! */
    }
    
    /* handle events of other sockets / established connections */
    for (i=0; i<=nMaxSock; ++i) {
      if (FD_ISSET(i, &tempset)) {
	/* handle data from socket i */
	nLength = recv(i, cstr, IBRC_MAXSTRSIZE, 0);
	if (nLength <= 0) {
	  /* Closed connection or error */
	  FD_CLR(i, &readset);
	  /* should check nMaxSock here ... */
	  /* do some cleaning */
	  shutdown(i, 2);
	  close(i);
	}
	else {
	  /* Process data in buffer */
	  printf("%d bytes read. ", nLength);
	  interpreteCommand(cstr, i);
	  /* write a message */
	  /* char *msg="MSG server DEFAULT All right. I handle your message.";
	     write(i, msg, strlen(msg)); */
	}
      }
    }
    /* handling done */

  }

  printf("Connection closed.\n");
  close(nMainSocketFd);
  return 0;
}

void interpreteCommand(char *cmd, int nSockFdFrom){
  char cstrCmd[IBRC_MAXSTRSIZE], *nick, *channel, *cstrToken;
  trChannel *prChannel;
  trClient *prClient;
  strncpy(cstrCmd, cmd, IBRC_MAXSTRSIZE);
  cstrToken=strtok(cmd, ibrc_DELIMS);
  
  if(cstrToken!=NULL){
    if(0==strcmp(cstrToken, ibrc_commands[IBRC_COMMAND_MSG])){
      cstrToken=strtok(NULL, ibrc_DELIMS); /* nick */
      nick = cstrToken;
      cstrToken=strtok(NULL, ibrc_DELIMS); /* channel */
      channel = cstrToken;
      cstrToken=strtok(NULL, ibrc_MSGEND); /* message */
      printf("MSG %s %s %s\n", nick, channel, cstrToken);
      sendToChannel(cstrCmd, channel, nSockFdFrom);
    }
    else if(0==strcmp(cstrToken, ibrc_commands[IBRC_COMMAND_JOIN])){
      cstrToken=strtok(NULL, ibrc_DELIMS); /* nick */
      nick = cstrToken;
      cstrToken=strtok(NULL, ibrc_MSGEND); /* channel */
      printf("JOIN %s %s\n",nick,cstrToken);
      prChannel = (trChannel *)hash_table_lookup(htChannels, cstrToken);
      if(prChannel==NULL) prChannel = addChannel(cstrToken);
      prClient = (trClient *)hash_table_lookup(htClients, nick);
      if(prClient==NULL) addClient(nick, prChannel, nSockFdFrom);
      else prClient->channel=prChannel;
      sendToAll(cstrCmd, nSockFdFrom);
    }
    else if(0==strcmp(cstrToken, ibrc_commands[IBRC_COMMAND_LEAVE])){
      /* ... */
    }
    else if(0==strcmp(cstrToken, ibrc_commands[IBRC_COMMAND_PRIVMSG])){
      /* ... */
    }
    else if(0==strcmp(cstrToken, ibrc_commands[IBRC_COMMAND_NICKCHANGE])){
      /* ... */
    }
    else if(0==strcmp(cstrToken, ibrc_commands[IBRC_COMMAND_TOPIC])){
      /* ... */
    }
    else if(0==strcmp(cstrToken, ibrc_commands[IBRC_COMMAND_ERROR])){
      /* ... */
    }
    else if(0==strcmp(cstrToken, ibrc_commands[IBRC_COMMAND_LISTREPLY])){

      /* ... */
    }
    else{
      /* ... */
    }
  }
}

trChannel *addChannel(char *channel){
  int size;
  char *newchannel, *newtopic;
  trChannel *prNewChannel;

  size = strlen(channel);
  if(size>IBRC_CHANNELLEN) size=IBRC_CHANNELLEN;
  newchannel = (char *)malloc(size*(sizeof(char)));
  newtopic = (char *)malloc(IBRC_TOPICLEN*(sizeof(char)));
  strncpy(newchannel, channel, size);
  prNewChannel = (trChannel *)malloc(sizeof(trChannel));
  prNewChannel->name = newchannel;
  prNewChannel->topic = newtopic;
  hash_table_insert(htChannels, newchannel, (void *)prNewChannel);
  printf("new Channel: %s\n", newchannel);
  return prNewChannel;
}


void addClient(char *nick, trChannel *channel, int nSocketFd){
  int size;
  char *newnick;
  trClient *prNewClient;
  size = strlen(nick);
  if(size>IBRC_NICKLEN) size=IBRC_NICKLEN;
  newnick = (char *)malloc(size*(sizeof(char)));
  strncpy(newnick, nick, size);
  prNewClient = (trClient *)malloc(sizeof(trClient));
  prNewClient->nick = newnick;
  prNewClient->channel=channel;
  prNewClient->nSockFd=nSocketFd;
  hash_table_insert(htClients, newnick, (void *)prNewClient);

  prNewClient=(trClient *)hash_table_lookup(htClients, newnick);
  printf("added %s [%s]\n", prNewClient->nick, prNewClient->channel->name);
}


void sendToChannel(char *packet, char *channel, int from){
  int i;
  trHashtable *pEntry;
  trClient *prClient;
  for(i=0; i<HASHTABLESIZE; ++i){
    pEntry = htClients[i];
    while(pEntry != NULL){
      prClient = (trClient *)pEntry->val;
      printf("send to Client: %s %s\n",prClient->nick, prClient->channel->name);
      if((prClient->nSockFd!=from) && (strcmp(channel, prClient->channel->name)==0)){
	write(prClient->nSockFd, packet, strlen(packet));
      }
      pEntry = pEntry->next;
    }
  }
}

void sendToAll(char *packet, int from){
  /* ... */
}
